<?php

/* 
    Evergreen Sheet Spine Label Converter
    Released to the public domain by John C. Rucker

     * Requires pdftotext, part of the poppler-utils package in Ubuntu.

    2008-08-28: v 0.1 - First release
    2012-06-11: v 0.2 - bugfix release

*/

// Change the value below to match the location of the pdftotext binary on your system
$pdftotext = "/usr/bin/pdftotext";

// processing the upload
if ( $_POST["submit"] == "true" )
{
    // only allow PDF files to be uploaded
    if ($_FILES["file"]["type"] == "application/pdf")
    {
        // stop on errors
        if ($_FILES["file"]["error"] > 0)
        {
            die("Error: " . $_FILES["file"]["error"]);
        }
        else // file is valid
        {
            // library of PDF functions
            include 'class.ezpdf.php';
            
            $upload = $_FILES["file"]["tmp_name"];
            
            // convert the Evergreen PDF to plain text
            $label_text = shell_exec("$pdftotext -nopgbrk $upload -");
            
            // put the individual barcodes into an array
            $text_array = explode("\n\n", $label_text); 

            // instantiate the exPDF object, US Letter size
            $pdf = new Cezpdf( 'letter', 'portrait' );
            
            // Choose font (see fonts directory for choices)
            $pdf->selectFont( './fonts/Helvetica-Bold' );
            
            // grab the label stock choice
            $label_stock = trim(urldecode($_POST["label_stock"]));
            
            // Demco 1-1/2x7/8
            if ($label_stock == "demco1")
            {
                // Demco1 margins
                $pdf->ezSetMargins(60, 60, 28, 28);
                
                // 8 columns, with an 14pt gap between them  
                $pdf->ezColumnsStart(array('num'=>8,'gap'=>14));   
                
                // grab the label start point
                $label_start = trim(urldecode($_POST["label_start"]));
                
                // fail if start point is out of allowed range
                if (!is_numeric($label_start) || ($label_start > 48 || $label_start < 1))
                {
                    die("Label Starting point must be between 1 and 48");
                }
                
                // pad the layout to start at the chosen label
                if ($label_start <=6)
                {
                    $cols_to_skip = 0;
                }
                else
                {
                    if ($label_start % 6 == 0)
                    {
                        $cols_to_skip = floor($label_start / 6) -1;
                    }
                    else
                    {
                        $cols_to_skip = floor($label_start / 6);
                    }
                }
                
                for ($i = 0; $i < $cols_to_skip; $i++ )
                {
                    $Dy = 0 - (117 * 5);
                    $pdf->ezSetDy($Dy);
                    $pdf->ezText(" ", 10);
                    $pdf->ezSetDy(-107);
                }
                
                $label_start = $label_start - ($cols_to_skip * 6);
                $Dy = (0 - 118.6) * ($label_start - 1);
                $pdf->ezSetDy($Dy);
                
                // write each label
                for ($i = 0; $i < count($text_array); $i++)
                {
                    // break up the call number into separate lines
                    $single_label = explode("\n", trim($text_array[$i]));
                    
                    // write each line
                    for ($j = 0; $j < count($single_label); $j++)
                    {
                        $pdf->ezText($single_label[$j], 10, array("leading"=>10));
                    }
                    
                    // move the cursor down to the next label
                    $Dy = 0 - (117 - count($single_label) * 10);
                    
                    $pdf->ezSetDy($Dy);
                }
                
                // test lines
                /*$pdf->setLineStyle(.5);
                $y = 47;
                for ($k = 0; $k < 7; $k++)
                {
                    $pdf->line(18, $y, 594, $y);
                    $y = $y + 117;
                }
                $x = 18;
                for ($k = 0; $k < 9; $k++)
                {
                    $pdf->line($x, 47, $x, 749);
                    $x = $x + 72;
                }*/
            }
            else
            {
                die("Unsupported label stock: $label_stock");
            }
            
            // stop columnar layout
            $pdf->ezColumnsStop();
            
            // generate the converted PDF
            $pdf->ezStream();
        }
    }
    else // fail if file uploaded is not a PDF
    {
        die("Invalid file type");
    }
}
else // viewing the upload interface
{    
echo "<?xml version=\"1.0\" encoding=\"ISO-8859-15\"?>";
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
    <head>
        <title>
            Spine Labels
        </title>
        <style type="text/css">
            a
            {
                color: #0000ff;
            }
            a:visited
            {
                color: #0000ff;
            }
            body
            {
                background: #ffffff;
                font-family: sans-serif;
                margin: 1em;
            }
            form
            {
                padding-top: 100px;
            }
            table
            {
                 text-align: center; 
                 width: 240px; 
                 height: 320px; 
                 float: left; 
                 margin-right: 50px;
            }
            td
            {
                width: 12.5%;
            }
            .cleared
            {
                background-color: transparent;
            }
            .marked
            {
                background-color: #00ff00;
            }
            .small
            {
                font-size: .75em;
            }
            #instructions
            {
                clear: both; 
                max-width: 640px; 
                padding-top: 2em;
                display: none;
            }
        </style>
    </head>
    <body>
    <h3>
        Evergreen Sheet Spine Label Converter
    </h3>
    <script type="text/javascript">
    <!--
        function toggle(element)
        // changes the background color of the element clicked on
        //    and sets the value of the label_start form input variable
        {
            var i;
            var tds = document.getElementsByTagName('td');
            
            for (i = 0; i < tds.length; i++)
            {
                tds[i].className = 'cleared';
            }
    
            element.className = 'marked';
            
            document.getElementById('label_start').value = element.innerHTML;
        }
        
        function toggle_instructions(element)
        // shows and hides the instructions
        {
            if (document.getElementById('instructions').style.display == 'block')
            {
                document.getElementById('instructions').style.display = 'none';
            }
            else
            {
                document.getElementById('instructions').style.display = 'block';
            }
            
            if (element.innerHTML == 'Show full instructions')
            {
                element.innerHTML = 'Hide instructions';
            }
            else
            {
                element.innerHTML = 'Show full instructions';
            }
        }
    //-->
    </script> 
    <table border="1" style="margin-bottom: 2em;">
        <caption>1) Click on a label number to change the starting label:</caption>
        <tr>
            <td class="marked" onclick="toggle(this);">1</td>
            <td class="cleared" onclick="toggle(this);">7</td>
            <td class="cleared" onclick="toggle(this);">13</td>
            <td class="cleared" onclick="toggle(this);">19</td>
            <td class="cleared" onclick="toggle(this);">25</td>
            <td class="cleared" onclick="toggle(this);">31</td>
            <td class="cleared" onclick="toggle(this);">37</td>
            <td class="cleared" onclick="toggle(this);">43</td>
        </tr>
        <tr>
            <td class="cleared" onclick="toggle(this);">2</td>
            <td class="cleared" onclick="toggle(this);">8</td>
            <td class="cleared" onclick="toggle(this);">14</td>
            <td class="cleared" onclick="toggle(this);">20</td>
            <td class="cleared" onclick="toggle(this);">26</td>
            <td class="cleared" onclick="toggle(this);">32</td>
            <td class="cleared" onclick="toggle(this);">38</td>
            <td class="cleared" onclick="toggle(this);">44</td>
        </tr>
        <tr>
            <td class="cleared" onclick="toggle(this);">3</td>
            <td class="cleared" onclick="toggle(this);">9</td>
            <td class="cleared" onclick="toggle(this);">15</td>
            <td class="cleared" onclick="toggle(this);">21</td>
            <td class="cleared" onclick="toggle(this);">27</td>
            <td class="cleared" onclick="toggle(this);">33</td>
            <td class="cleared" onclick="toggle(this);">39</td>
            <td class="cleared" onclick="toggle(this);">45</td>
        </tr>
        <tr>
            <td class="cleared" onclick="toggle(this);">4</td>
            <td class="cleared" onclick="toggle(this);">10</td>
            <td class="cleared" onclick="toggle(this);">16</td>
            <td class="cleared" onclick="toggle(this);">22</td>
            <td class="cleared" onclick="toggle(this);">28</td>
            <td class="cleared" onclick="toggle(this);">34</td>
            <td class="cleared" onclick="toggle(this);">40</td>
            <td class="cleared" onclick="toggle(this);">46</td>
        </tr>
        <tr>
            <td class="cleared" onclick="toggle(this);">5</td>
            <td class="cleared" onclick="toggle(this);">11</td>
            <td class="cleared" onclick="toggle(this);">17</td>
            <td class="cleared" onclick="toggle(this);">23</td>
            <td class="cleared" onclick="toggle(this);">29</td>
            <td class="cleared" onclick="toggle(this);">35</td>
            <td class="cleared" onclick="toggle(this);">41</td>
            <td class="cleared" onclick="toggle(this);">47</td>
        </tr>
        <tr>
            <td class="cleared" onclick="toggle(this);">6</td>
            <td class="cleared" onclick="toggle(this);">12</td>
            <td class="cleared" onclick="toggle(this);">18</td>
            <td class="cleared" onclick="toggle(this);">24</td>
            <td class="cleared" onclick="toggle(this);">30</td>
            <td class="cleared" onclick="toggle(this);">36</td>
            <td class="cleared" onclick="toggle(this);">42</td>
            <td class="cleared" onclick="toggle(this);">48</td>
            </tr>
        </table>

        <form action="<?php echo $_SERVER['PHP_SELF']; ?>" method="post" enctype="multipart/form-data">
            <p>
                2) Browse for a PDF to convert:
            </p>
            <input type="file" name="file" id="file" /><br /><br />
            <input type="hidden" id="label_start" name="label_start" value="1" />
            <input type="hidden" id="label_stock" name="label_stock" value="demco1" />
            <input type="hidden" id="submit" name="submit" value="true" />
            <input type="submit" value="Convert" />
        </form>
        <p class="small">
            <a href="#instructions" onclick="toggle_instructions(this);">Show full instructions</a>
        </p>
        <a name="instructions"></a>
        <div id="instructions">
            <h3>
                Instructions
            </h3>
            <p>
                This label generator will convert the single-page per label output from Evergreen into a sheet of Demco labels, (1-1/2"x7/8", 48 to a page). You can create other templates with a little effort.
            </p>
            <ol style="line-height: 1.5em;">
                <li>Queue up your labels like normal in Evergreen. (<a href="http://evergreen-ils.org/dokuwiki/doku.php?id=evergreen-user:printing_spine_labels#batch_printing">Look here</a> if you need a refresher.)</li>
                <li>If it's not already, you probably want to change "Label Width (in characters)" field to a number larger than the default.  20 would be fine. click the "Re-Generate" button after you do that.<br /><img src="eg1.png" alt="Evergreen Label Preview" /></li>
                <li>Click "Preview and Print", then "Print" again on the window that pops up. Choose "PDFCreator" as your printer on the print dialog box.<br/>
                    (You'll need to have <a href="http://sourceforge.net/projects/pdfcreator/">PDFCreator</a> installed, if it's not already.  Macintosh users can use the "Save as PDF" option on any print dialog box.)</li>
                <li>You'll need to make sure that you are not printing any headers or footers in your PDF output. Mac users can just toggle this on the print dialog box. Windows users may need to edit the prefs.js file in the OpenILS profile directory, making the value of the six header and footer lines like <code>user_pref("print.printer_PDFCreator.print_footercenter", "");</code> be empty strings. On Windows 7 this file is located at <code>C:\Users\YOUR-USER-NAME\AppData\Roaming\OpenILS\open_ils_staff_client\Profiles\SOME-RANDOM-STRING.default\prefs.js</code></li>
                <li>Click OK to print, and soon after the PDFCreator box will pop up. Uncheck "After saving open the document with the default program" and click "Save". Choose a location where you'll find it easily, like your Desktop.</li>
                <li>Click "Browse" above on this page to find your "Spine Labels.pdf" file, then click on "Open".</li>
                <li>If you are not starting with a fresh sheet of labels, click on the corresponding label number on the chart at the top of this page to set the starting point.</li>
                <li>Click "Convert".</li>
                <li>A new PDF file will be displayed for you.  You can now print that on your label paper in any printer.  When you print from Adobe Reader, or similar programs, make sure that "Page Scaling" is set to "None".</li>
            </ol>
        </div>
        <hr style="width: 50%; clear: both;" />
        <ul>
            <li class="small">
                v. 0.2: released on 2012-06-11<br />
                minor bugfix to address spacing issues<br />
                <a href="eg_spine_label_convertor_0_2.tar.bz2">Download the 0.2 source code</a>.<br /><br />
            </li> 
            <li class="small">
                v. 0.1: Initial release on 2008-08-28<br />
                <a href="eg_spine_label_convertor_0_1.tar.bz2">Download the 0.1 source code</a>.
            </li>
        </ul>
        <p class="small">
            Requirements are:
        </p>
        <ul class="small">
            <li>A Linux or Unix web server</li>
            <li>Apache/PHP installed</li>
            <li>the pdftotext utility, part of the poppler-utils package on Debian/Ubuntu systems</li>
            <li><a href="http://pdf-php.sourceforge.net/">Documentation for the R&OS PHP PDF creation toolkit is on it's Sourceforge page</a>.
        </ul>
    </body>
</html>

<?php
}
?>
